﻿using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.StoragesContracts;
using PrecastConcretePlantContracts.ViewModels;
using PrecastConcretePlantDataBaseImplement.Models;
using PrecastConcretePlantDataModels.Models;
using Microsoft.EntityFrameworkCore;

namespace PrecastConcretePlantDataBaseImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        public List<ShopViewModel> GetFullList()
        {
            using var context = new PrecastConcretePlantDataBase();
            return context.Shops
                    .Include(x => x.Reinforceds)
                    .ThenInclude(x => x.Reinforced)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            using var context = new PrecastConcretePlantDataBase();
            return context.Shops
                    .Include(x => x.Reinforceds)
                    .ThenInclude(x => x.Reinforced)
                    .Where(x => x.ShopName.Contains(model.ShopName))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new PrecastConcretePlantDataBase();
            return context.Shops
                .Include(x => x.Reinforceds)
                .ThenInclude(x => x.Reinforced)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) ||
                                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            using var context = new PrecastConcretePlantDataBase();
            var newShop = Shop.Create(context, model);
            if (newShop == null)
            {
                return null;
            }
            context.Shops.Add(newShop);
            context.SaveChanges();
            return newShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            using var context = new PrecastConcretePlantDataBase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var shop = context.Shops.FirstOrDefault(rec => rec.Id == model.Id);
                if (shop == null)
                {
                    return null;
                }
                shop.Update(model);
                context.SaveChanges();
                shop.UpdateReinforceds(context, model);
                transaction.Commit();
                return shop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public ShopViewModel? Delete(ShopBindingModel model)
        {
            using var context = new PrecastConcretePlantDataBase();
            var element = context.Shops
                .Include(x => x.Reinforceds)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Shops.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        //продажа изделий
        public bool MakeSale(IReinforcedModel model, int count)
        {
            using var context = new PrecastConcretePlantDataBase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                foreach (var shop in context.Shops.Include(x => x.Reinforceds).ThenInclude(x => x.Reinforced)
                    .Where(x => x.Reinforceds.Any(x => x.ReinforcedId == model.Id))
                    .ToList())
                {
                    var reinforced = shop.ShopReinforceds[model.Id];
                    int min = Math.Min(reinforced.Item2, count);
                    if (min == reinforced.Item2)
                    {
                        shop.ShopReinforceds.Remove(model.Id);
                    }
                    else
                    {
                        shop.ShopReinforceds[model.Id] = (reinforced.Item1, reinforced.Item2 - min);
                    }
                    shop.UpdateReinforceds(context, new() { Id = shop.Id, ShopReinforceds = shop.ShopReinforceds });
                    count -= min;
                    if (count == 0)
                    {
                        context.SaveChanges();
                        transaction.Commit();
                        return true;
                    }
                }
                transaction.Rollback();
                return false;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
    }
}
